(function(window, undefined){
    "use strict";

var scenemanager;
var drawables;

var currentCount = null;
var score = null;
var pause_overlay = null;
//var timer = null;
var addLineHint = null;

var beginTime;

var tuto = false;
var isFalling = false;

var manager = {

    MAX_COLUMN: 9,
    MAX_LINE: 8,

    cards: [],
    column_lens: [],
    spawner: null,

    is_game_paused: false,

    offset: [0,0],

    init: function()
    {
        for (var x = 0; x < manager.MAX_COLUMN; x++)
        {
            manager.column_lens.push(0);

            manager.cards[x] = [];
            for (var y = 0; y < manager.MAX_LINE; y++)
            {
                manager.cards[x][y] = null;
            }
        }

        if (!tuto)
        {
            manager.spawner = new Spawner(manager.spawn, scenemanager);
            manager.spawner.init();

            this.spawn();
        }
        else
        {
            manager.spawner = new Spawner(manager.spawn, scenemanager);
            this.spawn(true);
        }
    },

    update: function(delta)
    {
        var nbCards = 0;
        for (var x = 0; x < manager.MAX_COLUMN; x++)
        {
            for (var y = 0; y < manager.MAX_LINE; y++)
            {
                if (manager.cards[x][y])
                {
                    manager.cards[x][y].update(delta);
                    nbCards += 1;
                }
            }
        }

        if (isFalling)
        {
            return;
        }

        if (nbCards <= 7 && !tuto)
        {
            addLineHint.show();
        }
        else
        {
            addLineHint.hide();
        }

        if (!tuto && manager.spawner.play_time/manager.spawner.spawn_time >= Card.SHAKE_TIME)
        {
            var factor = (manager.spawner.play_time/manager.spawner.spawn_time - Card.SHAKE_TIME) / (1.0 - Card.SHAKE_TIME);

            manager.offset[0] = Math.random() * factor * Card.SHAKE_POWER;
            manager.offset[1] = Math.random() * factor * Card.SHAKE_POWER;
        }
        else
        {
            manager.offset[0] = 0;
            manager.offset[1] = 0;
        }


        manager.spawner.update(delta);
    },

    draw: function(canvas)
    {
        manager.spawner.draw(canvas);

        for (var x = 0; x < manager.MAX_COLUMN; x++)
        {
            for (var y = 0; y < manager.MAX_LINE; y++)
            {
                if (manager.cards[x][y])
                {
                    manager.cards[x][y].draw(canvas);
                }
            }
        }
    },

    makeFall: function()
    {
        for (var x = 0; x < manager.MAX_COLUMN; x++)
        {
            for (var y = 0; y < manager.MAX_LINE; y++)
            {
                if (manager.cards[x][y])
                {
                    manager.cards[x][y].isFalling = true;
                }
            }
        }
    },

    spawn: function spawn(cheat)
    {
        if (isFalling) return;

        cheat = cheat === undefined ? false : true;

        var newCard;
        for (var x = 0; x < manager.MAX_COLUMN; x++)
        {
            if (cheat && x >= 2 && x < 5)
            {
                newCard = new Card(scenemanager.get_asset('cards'), x, manager.column_lens[x]+1, 7, manager.offset);
            }
            else
            {
                newCard = new Card(scenemanager.get_asset('cards'), x, manager.column_lens[x]+1, undefined, manager.offset);
            }

            //newCard._onImageLoad();
            manager.add_to_column(newCard, x);
        }
    },

    add_to_column: function(card, x)
    {
        if (isFalling) return;

        manager.cards[x][manager.column_lens[x]] = card;
        manager.cards[x][manager.column_lens[x]].set_line(manager.column_lens[x]);
        manager.column_lens[x] += 1;
    },

    refresh_column: function(x)
    {
        var y;

        for (y = manager.column_lens[x]-1; y >=0 ; y--)
        {
            if (manager.cards[x][y] == null)
            {
                for (var i = y+1; i < manager.column_lens[x]; i++)
                {
                    manager.cards[x][i-1] = manager.cards[x][i];
                    manager.cards[x][i-1].set_line(i-1);
                }
                manager.cards[x][manager.column_lens[x]] = null;
                manager.column_lens[x] -= 1;
            }
        }
    },

    complete_line: function(is_same_color)
    {
        deck.cards = [];

        currentCount.set_text(0);
        return score.increment(is_same_color)
    },

    mousedown: function(e)
    {
        var found = false;

        if (manager.is_game_paused)
        {
            manager.pausegame();
            return;
        }

        if (deck.cards.length < manager.MAX_COLUMN)
        {
            for (var x = 0; x < manager.MAX_COLUMN; x++)
            {
                found = false;
                for (var y = 0; y < manager.column_lens[x]; y++)
                {
                    if (manager.cards[x][y] && manager.cards[x][y].rect.collidePoint(mouse.pos))
                    {
                        deck.add(manager.cards[x][y]);
                        manager.cards[x][y] = null;

                        found = true;
                        break;
                    }
                }
                if (found)
                {
                    manager.refresh_column(x);
                    break;
                }
            }
        }

        if (tuto)
        {
            tuto.nextStep(tuto.STEP_WIN);
        }

        if (found)
        {
            if (tuto)
            {
                tuto.nextStep(0);
            }

            deck.update_count();
            scenemanager.get_asset('blip').play();
        }
        else
        {
            deck.mousedown(e);
        }
    },

    show_all: function()
    {
        for (var x = 0; x < manager.cards.length; x++)
        {
            if (manager.cards[x])
            {
                for (var y = 0; y < manager.cards[x].length; y++)
                {
                    if (manager.cards[x][y])
                    {
                        manager.cards[x][y].alpha = 1;
                    }
                }
            }
        }

        for (var i = 0; i < deck.cards.length; i++)
        {
            deck.cards[i].alpha = 1;
        }
    },

    hide_all: function()
    {
        for (var x = 0; x < manager.cards.length; x++)
        {
            if (manager.cards[x])
            {
                for (var y = 0; y < manager.cards[x].length; y++)
                {
                    if (manager.cards[x][y])
                    {
                        manager.cards[x][y].alpha = 0;
                    }
                }
            }
        }

        for (var i = 0; i < deck.cards.length; i++)
        {
            deck.cards[i].alpha = 0;
        }
    },

    pausegame: function()
    {
        // unpause the game
        if (manager.is_game_paused)
        {
            manager.is_game_paused = false;

            manager.show_all();
            drawables.splice(drawables.indexOf(pause_overlay), 1);
        }
        // pause the game
        else
        {
            manager.is_game_paused = true;

            manager.hide_all();
            pause_overlay = new Sprite(scenemanager.get_asset('pause'));
            drawables.push(pause_overlay);
        }

        manager.spawner.toggle(manager.is_game_paused);
        //timer.toggle(manager.is_game_paused);
    }
};
window.manager = manager;

var deck = {

    cards: [],

    update: function (delta)
    {
        for (var i = 0; i < deck.cards.length; i++)
        {
            deck.cards[i].update(delta);
        }
    },

    draw: function (canvas)
    {
        for (var i = 0; i < deck.cards.length; i++)
        {
            deck.cards[i].draw(canvas);
        }
    },

    makeFall: function ()
    {
        for (var i = 0; i < deck.cards.length; i++)
        {
            deck.cards[i].isFalling = true;
        }
    },

    add: function(card)
    {
        deck.cards.push(card);
        deck.refresh_pos();
    },

    remove: function(i)
    {
        deck.cards.splice(i, 1);
        deck.refresh_pos();
    },

    refresh_pos: function()
    {
        for (var i = 0; i < deck.cards.length; i++)
        {
            //deck.cards[i].rect.pos.x = deck.cards[i].X_MARGIN + deck.cards[i].rect.size.x*i;
            deck.cards[i].set_pos(
                deck.cards[i].X_MARGIN + deck.cards[i].rect.size.x*i,
                //~ canvas_elem.height - deck.cards[i].rect.size.y - deck.cards[i].Y_MARGIN
                550 - deck.cards[i].rect.size.y - deck.cards[i].Y_MARGIN
            );  // change cc here on card click
        }
    },

    update_count: function()
    {
        var i;
        var count = 0;
        for (i = 0; i < deck.cards.length; i++)
        {
            count += deck.cards[i].number;
        }

        currentCount.set_text(count);

        if (count == 21)
        {
            deck.blackjack();
        }
        else if (count > 21)
        {
            _analytics.push('bad', 'count', count);
            if (tuto)
            {
                tuto.nextStep(1, tuto.STEP_LOSE);
            }
            else
            {
                manager.spawn();
            }

            for (i = deck.cards.length-1; i >= 0; i--)
            {
                deck.send_to_cards(i);
            }
            deck.update_count();

            score.decrement();
            scenemanager.get_asset('explode').play();

            drawables.push(
                new Bad(
                    scenemanager.get_asset('bad'),
                    drawables,
                    count
                )
            );
        }
    },

    blackjack: function()
    {
        // check color bonus
        var lastcolor = -1;
        var same_color = true;
        for (var i = 0; i < deck.cards.length; i++)
        {
            if(lastcolor >= 0 && lastcolor != deck.cards[i].color)
            {
                same_color = false;
                break;
            }
            else
            {
                lastcolor = deck.cards[i].color;
            }
        }

        _analytics.push('blackjack', 'same_color', same_color);
        if (same_color)
        {
            drawables.push(
                new ColorBonus(
                    scenemanager.get_asset('colorbonus'),
                    drawables
                )
            );
        }

        if (tuto)
        {
            tuto.nextStep(null, tuto.STEP_WIN);
        }

        var score_incr_val = manager.complete_line(same_color);

        // flying coins anim
        for (i = 0; i < score_incr_val; i++)
        {
            drawables.push(
                new Coin(
                    scenemanager.get_asset('coin'),
                    drawables
                )
            );
        }

        manager.spawner.reset();
        scenemanager.get_asset('positive').play();
    },

    send_to_cards: function(i)
    {
        //deck.cards[i].click();
        manager.add_to_column(deck.cards[i], deck.cards[i].column);
        deck.cards.splice(i, 1);
    },

    mousedown: function(e)
    {
        for (var i = 0; i < deck.cards.length; i++)
        {
            if (deck.cards[i].rect.collidePoint(mouse.pos))
            {
                deck.send_to_cards(i);
                deck.refresh_pos();
                scenemanager.get_asset('blip').play();
                break;
            }
        }

        deck.update_count();
    }
};
window.deck = deck;


function init(_drawables, _currentCount, _score/*, _timer*/, _scenemanager, _tuto)
{
    isFalling = false;
    drawables = _drawables;
    drawables.push(manager);
    drawables.push(deck);

    beginTime = new Date().getTime();

    scenemanager = _scenemanager
    currentCount = _currentCount;
    score = _score;
    //timer = _timer;

    addLineHint = new AddLineHint(scenemanager.get_asset('addLine'));
    drawables.push(addLineHint);

    tuto = _tuto || false;

    manager.init();

    if (!tuto)
    {
        events.on('keydown 32', function()
        {
            manager.spawner.reset();
            manager.spawner.decrement();
            manager.spawn();
        });

        events.on('keydown 27', function()
        {
            manager.pausegame();
        });

        if (window.kongregate)
        {
            kongregate.stats.submit("game start", 1);
        }
    }
    else
    {
        if (window.kongregate)
        {
            kongregate.stats.submit("tutorial start", 1);
        }
    }

    document.body.addEventListener("mousedown", manager.mousedown);
}

function destroy()
{
    manager.spawner.destroy();

    manager.cards = [];
    manager.column_lens = [];
    manager.spawner = null;

    deck.cards = [];
}

function saveScore()
{
    var playtime = parseInt(((new Date().getTime())-beginTime)/1000);

    if (window.kongregate)
    {
        kongregate.stats.submit("best score", score.count);
        kongregate.stats.submit("play time", playtime);
        kongregate.stats.submit("game end", 1);
    }
    else
    {
//      console.log('score:', score.count, 'play time:', playtime);
    }

    return score.count;
}

function makeFall()
{
    isFalling = true;

    events.remove('keydown 32');
    events.remove('keydown 27');
    document.body.removeEventListener("mousedown", manager.mousedown);

    manager.spawner.stop();

    manager.makeFall();
    deck.makeFall();
}

window.initCards = init;
window.destroyCards = destroy;
window.saveScore = saveScore;
window.makeFall = makeFall;

})(window);
