(function(window, undefined){
	"use strict";

var Sprite = function(src, pos, insize)
{
	Drawable.apply(this, []);
	
	this.src = src;
	this.rect = null;
	this.anim_column = 0;
	this.current_anim = 0;
	this.alpha = 1.0;
	this.angle = 0;
	
	this._pos = pos || new Vect();
	this._insize = insize || null;
	
	/* loading */
	if (this.src && this.src.charAt !== undefined) // is string
	{
		/*var self = this;
		this.img = new Image();
		this.img.addEventListener('load',
			function()
			{
				self._onImageLoad.apply(self);
			}
		);
		this.img.src = this.src;*/
	}
	else if(this.src && this.src.src !== undefined) // is img
	{
		this.img = src;
		this.src = this.img.src;
		this._onImageLoad();
	}
}
Sprite.prototype = new Drawable();
Sprite.prototype.constructor = Drawable;
Sprite.prototype._onImageLoad = function()
{
	if (this._insize == null)
	{
		this.rect = new Rect(
			this._pos,
			new Vect(this.img.width, this.img.height)
		);
	}
	else
	{
		this.rect = new Rect(
			this._pos,
			new Vect(this._insize.x, this._insize.y)
		);
	}
};
Sprite.prototype.draw = function(canvas)
{
	if (this.alpha == 0) return;
	
	if (this.img && this.rect && this.img.width > 0 && this.img.height > 0)
	{
		var need_save = this.angle != 0 || this.alpha != 1;
		if (need_save)
		{
			canvas.save();
		}
		
		// rortate the context
		if (this.angle != 0)
		{
			var tr = new Vect(
				this.rect.pos.x + (this.rect.size.x/2),
				this.rect.pos.y + (this.rect.size.y/2)
			);
			
			//put the rotation point at the center of the sprite
			canvas.translate(
				tr.x,
				tr.y
			);
			
			//rotate by angle
			canvas.rotate(this.angle * Math.PI /180);
			
			//put the rotation points back to the origins
			canvas.translate(
				-tr.x,
				-tr.y
			);
		}
		
		// the the alpha
		if (this.alpha != 1)
		{
			canvas.globalAlpha = this.alpha;
		}
		
		if (canvas.drawImage == undefined)
		{
			console.log('canvas is null', canvas, this);
		}
		
		
		// blit
		canvas.drawImage(
			this.img, //image
			this.rect.size.x*this.anim_column, //sx
			this.rect.size.y*this.current_anim, //sy
			this.rect.size.x, //sw
			this.rect.size.y, //sh
			this.rect.pos.x, //dx
			this.rect.pos.y, //dy
			this.rect.size.x, //dw
			this.rect.size.y //dy
		);
		
		// restore the context
		if (need_save)
		{
			canvas.restore();
		}
	}
}

Sprite.prototype.update = function(delta)
{
	/*if (this.speed.x != 0 || this.speed.y != 0)
	{
		this.move(this.speed.x, this.speed.y);
	}*/
}

window.Sprite = Sprite;

})(window);
