(function(window, undefined){
    "use strict";

var Card = function(image, column, line, number, offset)
{
    this.X_MARGIN = 30;
    this.Y_MARGIN = 35;
    this.MAX_NUM = 12;
    this.MAX_LINE = 7;

    this.EASE_TIME = 500;

    this.CARD_WIDTH = image.width/12;
    this.CARD_HEIGHT = image.height/2;

    this.FALLING_INIT_SPEED_X = 0.15;
    this.FALLING_INIT_SPEED_Y = -0.3;
    this.FALLING_GRAVITY  = 0.02;
    this.FALLING_Y_MAX  = 0.4;
    this.FALLING_ROTATE = 0.9;

    Sprite.apply(this, [
        image,
        new Vect(
            this.X_MARGIN + this.CARD_WIDTH*column,
            this.Y_MARGIN + line*this.CARD_HEIGHT
        ),
        new Vect(
            this.CARD_WIDTH,
            this.CARD_HEIGHT
        )
    ]);

    this.line = line;
    this.column = column;
    this.offset = offset;

    if (number !== undefined)
    {
        this.number = number;
    }
    else
    {
        this.number = parseInt((Math.random() * this.MAX_NUM) +1);
    }

    if (this.number >= 10)
    {
        this.drawNumber = 10 + parseInt(Math.random() * (this.MAX_NUM - 9));
        this.number = 10;
    }
    else
    {
        this.drawNumber = this.number;
    }


    this.anim_column = this.drawNumber-1;

    this.color = Math.round(Math.random()*1)
    this.current_anim = this.color;

    // the falling anim
    this.isFalling = false;
    this.falling = {

        speed: [
            Math.random()*this.FALLING_INIT_SPEED_X*2 - this.FALLING_INIT_SPEED_X,
            this.FALLING_INIT_SPEED_Y
        ]
    };
    this.falling.angleTarget = Math.PI*2 * this.falling.speed[0];

    this.easing;
}

Card.SHAKE_TIME = 0.75; //in percent
Card.SHAKE_POWER = 5;  //in px

Card.prototype = new Sprite();
Card.prototype.constructor = Sprite;
Card.prototype.draw = function(canvas)
{
    var x = Math.round(this.rect.pos.x);
    var y = Math.round(this.rect.pos.y);

    this.rect.pos.x = x - this.offset[0];
    this.rect.pos.y = y - this.offset[1];

    Sprite.prototype.draw.apply(this, [canvas]);

    this.rect.pos.x = x;
    this.rect.pos.y = y;
}
Card.prototype.update = function(delta)
{
    if (this.isFalling)
    {
        this.rect.pos.x = Math.round(this.rect.pos.x + delta * this.falling.speed[0]);
        this.rect.pos.y = Math.round(this.rect.pos.y + delta * this.falling.speed[1]);

        this.falling.speed[1] = Math.min(this.FALLING_Y_MAX, this.falling.speed[1]+this.FALLING_GRAVITY);

        this.angle += this.FALLING_ROTATE*this.falling.angleTarget;
    }
    else
    {
        if (this.easing) this.easing.update(delta);
    }
}
Card.prototype.set_line = function(_line)
{
    this.line = _line;

    if (this.line < this.MAX_LINE)
    {
        this.set_pos(
            this.X_MARGIN + this.rect.size.x*this.column,
            this.Y_MARGIN + this.rect.size.y*this.line
        );
    }
    else
    {
        events.emit('lose');
    }
}
Card.prototype.set_pos = function(_x, _y)
{
    var x = _x/* || this.rect.pos.x*/;
    var y = _y/* || this.rect.pos.y*/;

    this.easing = new Ease(this.rect.pos, this.EASE_TIME, {x:x, y:y});
}


window.Card = Card;

})(window);
