(function(window, undefined){
    "use strict";

var GameScene = function(manager)
{
    this.manager = manager;
    this.drawables = [];

    _analytics.push('startPlay');

    //background
    var background = new Sprite(manager.get_asset('gamescene'));
    this.drawables.push(background);

    //current count
    var currentCount = new Count();
    this.drawables.push(currentCount);

    //score
    var score = new Score();
    this.drawables.push(score);

    //cards
    initCards(this.drawables, currentCount, score/*, timer*/, manager);

    //add lose events
    var self = this;
    this._loseAnim = function(){self.loseAnim.apply(self)};
    events.on('lose', this._loseAnim);
    
    // pause btn
    events.on('NeedPause', this.onNeedPause);
    this.pausebtn = new Button(
            manager.get_asset('pauseBtn'),
            new Vect(
                38,
                0
            ),
            this.onNeedPause
        );
}
GameScene.prototype.destroy = function()
{
    destroyCards();
    this.drawables = [];
    events.remove('NeedPause', this.onNeedPause);
}
GameScene.prototype.update = function(delta)
{
    for (var i = 0; i < this.drawables.length; i++)
    {
        this.drawables[i].update(delta);
    }
    
    if (this.pausebtn) this.pausebtn.update(delta);
};
GameScene.prototype.draw = function(canvas)
{
    //~ stats.begin();
    
    for (var i = 0; i < this.drawables.length; i++)
    {
        this.drawables[i].draw(canvas);
    }
    
    if (!manager.is_game_paused && this.pausebtn)
        this.pausebtn.draw(canvas);

    //~ stats.end();
};
GameScene.prototype.loseAnim = function(canvas)
{
    makeFall();

    this.end_of_game();

    var self = this;
    setTimeout(function(){self.lose.apply(self)}, 2600);
};
GameScene.prototype.lose = function(canvas)
{
    var score = saveScore();

    //analytics.push('die', 'time', Math.round(this.timer.val/1000));

    this.manager.change('end', score);
};
GameScene.prototype.end_of_game = function(canvas)
{
    events.remove('lose', this._lose);
};
GameScene.prototype.onNeedPause = function(){
    if (!manager.is_game_paused)
        manager.pausegame();
};

window.GameScene = GameScene;

})(window);
