(function(window, undefined){
    "use strict";

var MenuScene = function(manager)
{
    var self = this;
    
    self.is_online = false;
    if (navigator.onLine)
    {
        var xhttp = new XMLHttpRequest();
        xhttp.onreadystatechange = function() {
            if (xhttp.readyState == 4 && xhttp.status >= 200 && xhttp.status < 400 ) {
                ads.init('ca-app-pub-6667465120048697/5487689167');
                self.is_online = true;
            }
        };
        xhttp.open("GET", "http://google.com", true);
        xhttp.send();
        
        _analytics.init();
        _analytics.push('loadGame');
    }
    
    //background
    this.background = new Sprite(manager.get_asset('menuscene'));

    this.isFirstTime = localStorage.getItem(MenuScene.FIRSTTIME_KEY) != '0';
    if (this.isFirstTime)
    {
        // the play BTN but it go to tuto
        this.playbtn = new Button(
            manager.get_asset('playBtn'),
            new Vect(
                canvas_elem.width - 160,
                canvas_elem.height - 100
            ),
            function(){
                self.play_outro_anim.call(
                    self,
                    function(){self.goto_tuto.apply(self);}
                );
            }
        );
        this.tutobtn = null;
    }
    else
    {
        // the tuto BTN
        this.tutobtn = new Button(
            manager.get_asset('tutoBtn'),
            new Vect(
                -170,
                canvas_elem.height - 90
            ),
            function(){
                self.play_outro_anim.call(
                    self,
                    function(){self.goto_tuto.apply(self);}
                );
            }
        );
        // the play BTN
        this.playbtn = new Button(
            manager.get_asset('playBtn'),
            new Vect(
                canvas_elem.width + 10,
                canvas_elem.height - 90
            ),
            function(){
                self.play_outro_anim.call(
                    self,
                    function(){self.goto_play.apply(self);}
                );
            }
        );
    }

    this.start_time = Date.now();
    this.play_btn_ease = new Ease(
        this.playbtn.rect.pos,
        MenuScene.INTRO_EASE_TIME,
        {x: canvas_elem.width - 160}
    );
    this.play_btn_ease.compute_val = function(val, percent){
        if (Date.now() - this.start_time < MenuScene.INTRO_EASE_WAIT)
            return 0
        else
            return val * Math.pow(percent, 3);
    }
    
    this.tutobtn_ease = null;
    if (this.tutobtn)
    {
        this.tutobtn_ease = new Ease(
            this.tutobtn.rect.pos,
            MenuScene.INTRO_EASE_TIME,
            {x: 45}
        );
        
        this.tutobtn_ease.compute_val = this.play_btn_ease.compute_val;
    }

    this.manager = manager;
};

MenuScene.FIRSTTIME_KEY = 'MathJackFirstTime';
MenuScene.INTRO_EASE_WAIT = 200;
MenuScene.INTRO_EASE_TIME = 800;
MenuScene.OUTRO_EASE_TIME = 400;

MenuScene.prototype.destroy = function()
{
    events.remove('keydown 27');
    this.playbtn.destroy();
};
MenuScene.prototype.update = function(delta)
{
    if (this.play_btn_ease) this.play_btn_ease.update(delta);
    if (this.tutobtn_ease) this.tutobtn_ease.update(delta);
};
MenuScene.prototype.draw = function(canvas)
{
    this.background.draw(canvas);
    this.playbtn.draw(canvas);
    if (this.tutobtn) this.tutobtn.draw(canvas);

    if (window.isloaded)
        canvas.fillStyle = "rgba(0,255,0,0.15)";
    else
        canvas.fillStyle = "rgba(255,0,0,0.15)";
    canvas.fillRect(10,10, 2, 2);

    if (window._analytics.is_init)
        canvas.fillStyle = "rgba(0,255,0,0.15)";
    else
        canvas.fillStyle = "rgba(255,0,0,0.15)";
    canvas.fillRect(20,10, 2, 2);

    if (ads.is_init)
        canvas.fillStyle = "rgba(0,255,0,0.15)";
    else
        canvas.fillStyle = "rgba(255,0,0,0.15)";
    canvas.fillRect(30,10, 2, 2);
    
    if (window.AdMob)
        canvas.fillStyle = "rgba(0,255,0,0.15)";
    else
        canvas.fillStyle = "rgba(255,0,0,0.15)";
    canvas.fillRect(40,10, 2, 2);
    
    if (this.is_online)
        canvas.fillStyle = "rgba(0,255,0,0.15)";
    else
        canvas.fillStyle = "rgba(255,0,0,0.15)";
    canvas.fillRect(50,10, 2, 2);
};
MenuScene.prototype.play_outro_anim = function(cb)
{
    this.play_btn_ease = new Ease(
        this.playbtn.rect.pos,
        MenuScene.OUTRO_EASE_TIME,
        {x: canvas_elem.width + 10},
        cb
    );

    if (this.tutobtn)
    {
        this.tutobtn_ease = new Ease(
            this.tutobtn.rect.pos,
            MenuScene.OUTRO_EASE_TIME,
            {x: -170}
        );
    }
};
MenuScene.prototype.goto_tuto = function()
{
    this.manager.change('tuto');
    this.manager.get_asset('blip').play();
};
MenuScene.prototype.goto_play = function()
{
    this.manager.change('game');
    this.manager.get_asset('blip').play();
};

window.MenuScene = MenuScene;

})(window);