(function(window, undefined){
    "use strict";

var Spawner = function(callback, manager)
{
    this.BASE = 30*1000;
    this.MIN = 6*1000;
    this.NB_TO_MAX = 7;
    this.CURV_LVL = 2;

    this.DECR_BONUS = 2000; // in ms

    this.nb_spawn = 0;
    this.play_time = 0;
    this.spawn_time = 0;

    this.spawner_id;
    this.callback = callback;

    this.gauge = new Sprite(manager.get_asset('gauge'),
        new Vect(
            16,
            421
        )
    );

    this.level_text = new Text(
        "level 0",
        new Rect(
            new Vect(
                this.gauge.rect.size.x - 75,
                458
            ),
            new Vect(0, 26)
        )
    );
    this.level_text.style.fillStyle = '#333';
}
Spawner.prototype.update = function(delta){}
Spawner.prototype.updateTime = function()
{
    this.play_time += 1000;
    if (this.play_time >= this.spawn_time)
    {
        this.callback();

        this.nb_spawn += 1;
        this.play_time = 0;

        this.level_text.set_text("level " + this.nb_spawn);

        if (this.nb_spawn >= this.NB_TO_MAX)
        {
            this.spawn_time = this.MIN;
        }
        else
        {
            this.spawn_time = this.BASE - (
                this.nb_spawn *
                Math.pow(this.nb_spawn/this.NB_TO_MAX, this.CURV_LVL) *
                ((this.BASE-this.MIN)/this.NB_TO_MAX)
            );
        }
    }
}
Spawner.prototype.init = function()
{
    this.spawn_time = this.BASE;
    this.start();
}
Spawner.prototype.start = function()
{
    var self = this;
    this.spawner_id = setInterval(
        function(){ self.updateTime.apply(self) },
        1000
    );
}
Spawner.prototype.toggle = function(is_paused)
{
    if (!is_paused)
    {
        this.start();
    }
    else
    {
        this.stop();
    }
};
Spawner.prototype.reset = function()
{
//  this.stop();
//  this.init();

//  this.play_time = 0;
};
Spawner.prototype.decrement = function()
{
    this.play_time = Math.max(0, this.play_time-this.DECR_BONUS)
};
Spawner.prototype.stop = function()
{
    clearInterval(this.spawner_id);
};
Spawner.prototype.draw = function(canvas)
{
    var COLOR = '#73d216';
    var LEFT_MARGIN = 18;
    var SIZE = 445;
    var POS_Y = this.gauge.rect.pos.y + 2;
    var HEIGHT = 6;

    var percent = this.play_time/this.spawn_time;
    var sizeVal = SIZE * percent;

    canvas.fillStyle = COLOR;
    canvas.fillRect(LEFT_MARGIN, POS_Y, sizeVal, HEIGHT);

    this.gauge.draw(canvas);
    this.level_text.draw(canvas);
};
Spawner.prototype.destroy = function()
{
    this.stop();
};

window.Spawner = Spawner;

})(window);
