(function(window, undefined){
    "use strict";

var SceneManager = function(scenes, canvas)
{
    this.canvas = canvas;
    this.scenes = scenes;
    this.current_id;
    this.current = null;

    this.needChange = -1;
    this.newSceneParams;

    this.interval_id;

    this.loader;
}
SceneManager.prototype.init_loader = function(assets)
{
    this.loader = new Loader(assets);
}
SceneManager.prototype.get_asset = function(key)
{
    return this.loader.assets[key];
}
SceneManager.prototype.start = function(canvas, current_id, assets)
{
    this.current_id = current_id;

    this.init_loader(assets);

    this.current = new this.scenes[this.current_id](this);

    //set update
    var self = this;
    this.lastUpdate = new Date().getTime();
    this.interval_id = window.setInterval(function(){self.update.apply(self, [self.canvas])}, 1000/60);

    //set draw
    requestAnimationFrame(function(){self.draw.apply(self, [self.canvas])});
}
SceneManager.prototype.update = function(canvas)
{
    if (this.needChange < 0)
    {
        var delta = this.lastUpdate = new Date().getTime() - this.lastUpdate;

        //console.log(delta);

        this.current.update(delta);
        mouse.is_changing = false;

        this.lastUpdate = new Date().getTime();
    }
    else
    {
        this.current_id = this.needChange;
        this.needChange = -1;

        this.current.destroy();
        //this.current = null;
        this.current = new this.scenes[this.current_id](this, this.newSceneParams);

        this.newSceneParams = null;
        mouse.is_changing = false;
    }
}
SceneManager.prototype.draw = function(canvas)
{
    var self = this;
    requestAnimationFrame(function(){self.draw.apply(self, [canvas])});

    canvas.clearRect(0,0,canvas_elem.width, canvas_elem.height);
    this.current.draw(canvas);
}
SceneManager.prototype.change = function(scene_id, newSceneParams)
{
    this.needChange = scene_id;
    this.newSceneParams = newSceneParams;
}

window.SceneManager = SceneManager;

})(window);
