(function(window, undefined){
    "use strict";

var EndScene = function(manager, score)
{
    this.COOKIE_KEY = 'mathsjackhightscore';

    this.score = score;
    this.best_score = null;
    this.is_hightscore = this.update_hightscore();

    _analytics.push('die', 'score', this.score);

    this.background = new Sprite(manager.get_asset('endscene'));

    var score_str = '';
    if (this.is_hightscore)
    {
        score_str = "New High Score!\n" +
            "score: "+this.score;
    }
    else
    {
        score_str = "score: "+this.score+"\n"+"best: "+this.best_score;
    }

    this.score_text = new Text(
        score_str,
        new Rect(
            new Vect(
                canvas_elem.width/2 - 100,
                this.background.rect.size.y / 2 + 100
            ),
            new Vect(0, 30)
        )
    );
    this.score_text.style.fillStyle = '#333';
    this.score_text.style.globalAlpha = 0.0;

    this.score_text_ease = new Ease(
        this.score_text.style,
        EndScene.INTRO_EASE_TIME / 2,
        {globalAlpha: 1.0}
    );

    // the play BTN
    var self = this;
    this.playbtn = new Button(
        manager.get_asset('playBtn'),
        new Vect(
            canvas_elem.width + 10,
            canvas_elem.height - 100
        ),
        function(){
            self.play_btn_pressed.apply(self);
        }
    );
    this.playbtn_ease = new Ease(
        this.playbtn.rect.pos,
        EndScene.INTRO_EASE_TIME,
        {x: canvas_elem.width - 160}
    );

    ads.show_banner();

    this.manager = manager;
};

EndScene.INTRO_EASE_TIME = 500;
EndScene.OUTRO_EASE_TIME = 300;

EndScene.prototype.get_hightscore = function()
{
    return parseInt(localStorage.getItem(this.COOKIE_KEY)) || 0;
};
EndScene.prototype.set_hightscore = function(best_score)
{
    localStorage.setItem(this.COOKIE_KEY, best_score);
};
EndScene.prototype.update_hightscore = function()
{
    var is_best = false;
    this.best_score = this.get_hightscore();

    if (this.score > this.best_score)
    {
        this.set_hightscore(this.score);
        is_best = true;
    }

    return is_best;
};
EndScene.prototype.destroy = function()
{
    this.playbtn.destroy();

    ads.hide_banner();
};
EndScene.prototype.update = function(delta)
{
    this.score_text_ease.update(delta);
    this.playbtn_ease.update(delta);
};
EndScene.prototype.draw = function(canvas)
{
    this.background.draw(canvas);
    this.score_text.draw(canvas);
    this.playbtn.draw(canvas);
};
EndScene.prototype.play_btn_pressed = function()
{
    var self = this;

    // you can't click until the intro anim finish
    if (!this.playbtn_ease.is_end)
    {
        return
    }

    this.manager.get_asset('blip').play();
    this.playbtn.enable = false;

    this.playbtn_ease = new Ease(
        this.playbtn.rect.pos,
        EndScene.OUTRO_EASE_TIME,
        {x: canvas_elem.width + 10},
        function() {self.goto_play.apply(self);}
    );
};
EndScene.prototype.goto_play = function()
{
    _analytics.push('replay');
    this.manager.change('game');
};


window.EndScene = EndScene;

})(window);
